@@ -3,10 +3,10 @@
 - support TextPattern, ElementPattern, LinePattern, BlockPattern, and PatternBuilder
 - support predefine pattern reference on system_references.yaml
 - allow end-user to customize pattern on /home/.regexapp/user_references.yaml
-- allow end-user to generate on GUI application and verify it.
+- allow end-user to generate test script or pattern on GUI application.
+- dynamically generate Python snippet script
 - dynamically generate Python unittest script
 - dynamically generate Python pytest script
-- dynamically generate Robotframework test script
 """
 
 from regexapp.collection import TextPattern
@@ -20,7 +20,7 @@ from regexapp.core import DynamicGenTestScript
 from regexapp.core import add_reference
 from regexapp.core import remove_reference
 
-__version__ = '0.1.0'
+__version__ = '0.1.1'
 version = __version__
 __edition__ = 'Community'
 edition = __edition__
