@@ -13,6 +13,7 @@ from regexapp.collection import REF
 from regexapp.collection import PatternReference
 from regexapp import version
 from regexapp import edition
+from regexapp.core import enclose_string
 
 import yaml
 import re
@@ -187,8 +188,13 @@ class Application:
         self.text_frame = None
         self.entry_frame = None
         self.result_frame = None
+        self.save_as_btn = None
+        self.copy_text_btn = None
 
-        self.radio_btn_var = tk.StringVar()
+        self.test_data = ''
+
+        self.radio_line_or_block_btn_var = tk.StringVar()
+        self.radio_line_or_block_btn_var.set('block')
         self.used_space_var = tk.BooleanVar()
         self.used_space_var.set(True)
         self.prepended_ws_var = tk.BooleanVar()
@@ -207,7 +213,6 @@ class Application:
         self.company_var = tk.StringVar()
 
         self.new_pattern_name_var = tk.StringVar()
-        self.result = None
 
         self.textarea = None
         self.result_textarea = None
@@ -221,6 +226,46 @@ class Application:
         self.build_entry()
         self.build_result()
 
+    def get_pattern_args(self):
+        """return arguments of RegexBuilder class"""
+        result = dict(
+            used_space=self.used_space_var.get(),
+            ignore_case=self.ignore_case_var.get(),
+            prepended_ws=self.prepended_ws_var.get(),
+            appended_ws=self.appended_ws_var.get(),
+            is_line=self.radio_line_or_block_btn_var.get() == 'line'
+        )
+        return result
+
+    def get_builder_args(self):
+        """return arguments of DynamicGenTestScript class"""
+        result = dict(
+            test_name=self.test_name_var.get(),
+            test_cls_name=self.test_cls_name_var.get(),
+            is_minimal=self.is_minimal_var.get(),
+            max_words=self.max_words_var.get(),
+            filename=self.filename_var.get(),
+            author=self.author_var.get(),
+            email=self.email_var.get(),
+            company=self.company_var.get()
+        )
+        return result
+
+    def set_default_setting(self):
+        """reset to default setting"""
+        self.used_space_var.set(True)
+        self.prepended_ws_var.set(False)
+        self.appended_ws_var.set(False)
+        self.ignore_case_var.set(False)
+        self.test_name_var.set('')
+        self.test_cls_name_var.set('TestDynamicGenTestScript')
+        self.is_minimal_var.set(True)
+        self.max_words_var.set(6)
+        self.filename_var.set('')
+        self.author_var.set('')
+        self.email_var.set('')
+        self.company_var.set('')
+
     @classmethod
     def get_textarea(cls, node):
         """Get data from TextArea component
@@ -283,6 +328,7 @@ class Application:
         if filename:
             with open(filename) as stream:
                 content = stream.read()
+                self.test_data = content
                 self.set_textarea(self.textarea, content, title=filename)
 
     def callback_help_documentation(self):
@@ -355,7 +401,7 @@ class Application:
 
         settings = tk.Toplevel(self.root)
         self.set_title(node=settings, title='Settings')
-        width, height = 400, 400
+        width, height = 400, 430
         x, y = get_relative_center_location(self.root, width, height)
         settings.geometry('{}x{}+{}+{}'.format(width, height, x, y))
         settings.resizable(False, False)
@@ -378,45 +424,53 @@ class Application:
             ttk.Checkbutton(lframe_pattern_args, text=text, variable=variable,
                             onvalue=True, offvalue=False).place(x=x, y=y)
 
-        # Settings - Regexapp Arguments
-        lframe_regexapp_args = ttk.LabelFrame(
+        # Settings - Builder Arguments
+        lframe_builder_args = ttk.LabelFrame(
             settings, height=210, width=380,
-            text='Regexapp Arguments'
+            text='Builder Arguments'
         )
-        lframe_regexapp_args.place(x=10, y=95)
+        lframe_builder_args.place(x=10, y=95)
 
-        ttk.Label(lframe_regexapp_args, text='max_words').place(x=5, y=5)
-        ttk.Entry(lframe_regexapp_args, width=5,
+        ttk.Label(lframe_builder_args, text='max_words').place(x=5, y=5)
+        ttk.Entry(lframe_builder_args, width=5,
                   textvariable=self.max_words_var).place(x=88, y=5)
 
-        ttk.Checkbutton(lframe_regexapp_args, text='is_minimal',
+        ttk.Checkbutton(lframe_builder_args, text='is_minimal',
                         variable=self.is_minimal_var,
                         onvalue=True, offvalue=False).place(x=200, y=5)
 
-        ttk.Label(lframe_regexapp_args, text='test_name').place(x=5, y=30)
-        ttk.Entry(lframe_regexapp_args, width=45,
+        ttk.Label(lframe_builder_args, text='test_name').place(x=5, y=30)
+        ttk.Entry(lframe_builder_args, width=45,
                   textvariable=self.test_name_var).place(x=88, y=30)
 
-        ttk.Label(lframe_regexapp_args, text='test_cls_name').place(x=5, y=55)
-        ttk.Entry(lframe_regexapp_args, width=45,
+        ttk.Label(lframe_builder_args, text='test_cls_name').place(x=5, y=55)
+        ttk.Entry(lframe_builder_args, width=45,
                   textvariable=self.test_cls_name_var).place(x=88, y=55)
 
-        ttk.Label(lframe_regexapp_args, text='filename').place(x=5, y=80)
-        ttk.Entry(lframe_regexapp_args, width=45,
+        ttk.Label(lframe_builder_args, text='filename').place(x=5, y=80)
+        ttk.Entry(lframe_builder_args, width=45,
                   textvariable=self.filename_var).place(x=88, y=80)
 
-        ttk.Label(lframe_regexapp_args, text='author').place(x=5, y=105)
-        ttk.Entry(lframe_regexapp_args, width=45,
+        ttk.Label(lframe_builder_args, text='author').place(x=5, y=105)
+        ttk.Entry(lframe_builder_args, width=45,
                   textvariable=self.author_var).place(x=88, y=105)
 
-        ttk.Label(lframe_regexapp_args, text='email').place(x=5, y=130)
-        ttk.Entry(lframe_regexapp_args, width=45,
+        ttk.Label(lframe_builder_args, text='email').place(x=5, y=130)
+        ttk.Entry(lframe_builder_args, width=45,
                   textvariable=self.email_var).place(x=88, y=130)
 
-        ttk.Label(lframe_regexapp_args, text='company').place(x=5, y=155)
-        ttk.Entry(lframe_regexapp_args, width=45,
+        ttk.Label(lframe_builder_args, text='company').place(x=5, y=155)
+        ttk.Entry(lframe_builder_args, width=45,
                   textvariable=self.company_var).place(x=88, y=155)
 
+        ttk.Button(settings, text='Default',
+                   command=lambda: self.set_default_setting(),
+                   width=8).place(x=270, y=395)
+
+        ttk.Button(settings, text='OK',
+                   command=lambda: settings.destroy(),
+                   width=8).place(x=332, y=395)
+
         set_modal_dialog(settings)
 
     def callback_preferences_system_reference(self):
@@ -658,119 +712,223 @@ class Application:
 
     def build_entry(self):
         """Build input entry for regex GUI."""
-        def callback_run_btn():
-            user_data = self.__class__.get_textarea(self.textarea)
+        def callback_build_btn():
+            user_data = Application.get_textarea(self.textarea)
             if not user_data:
                 create_msgbox(
                     title='Empty Data',
-                    error="Can not build regex pattern without data."
+                    error="Can NOT build regex pattern without data."
                 )
                 return
 
-            is_line = self.radio_btn_var.get() == 'line'
-            factory = RegexBuilder(user_data=user_data, is_line=is_line)
-            factory.build()
+            try:
+                kwargs = self.get_pattern_args()
+                factory = RegexBuilder(user_data=user_data, **kwargs)
+                factory.build()
 
-            lst = []
-            for user_data, pattern in factory.user_data_pattern_table.items():
-                lst.append('# {}'.format('-' * 10))
-                lst.append('# user data      : {}'.format(user_data))
-                lst.append('# created pattern: {}'.format(pattern))
+                patterns = factory.patterns
+                total = len(patterns)
+                if total >= 1:
+                    if total == 1:
+                        result = 'pattern = r{}'.format(enclose_string(patterns[0]))
+                    else:
+                        lst = []
+                        fmt = 'pattern{} = r{}'
+                        for index, pattern in enumerate(patterns, 1):
+                            lst.append(fmt.format(index, enclose_string(pattern)))
+                        result = '\n'.join(lst)
+                    self.set_textarea(self.result_textarea, result)
+                    self.save_as_btn.config(state=tk.NORMAL)
+                    self.copy_text_btn.config(state=tk.NORMAL)
+                else:
+                    error = 'Something wrong with RegexBuilder.  Please report bug.'
+                    create_msgbox(title='RegexBuilder Error', error=error)
+            except Exception as ex:
+                error = '{}: {}'.format(type(ex).__name__, ex)
+                create_msgbox(title='RegexBuilder Error', error=error)
 
-            result = '\n'.join(lst)
-            self.set_textarea(self.result_textarea, result)
+        def callback_save_as_btn():
+            filename = filedialog.asksaveasfilename()
+            if filename:
+                with open(filename, 'w') as stream:
+                    content = Application.get_textarea(self.result_textarea)
+                    stream.write(content)
 
         def callback_clear_text_btn():
             self.textarea.delete("1.0", "end")
             self.result_textarea.delete("1.0", "end")
-            self.result = None
+            self.save_as_btn.config(state=tk.DISABLED)
+            self.copy_text_btn.config(state=tk.DISABLED)
+            self.test_data = ''
+            # self.root.clipboard_clear()
             self.set_title()
 
-        def callback_paste_text_btn():
-            data = self.root.clipboard_get()
-            if not data:
-                return
+        def callback_copy_text_btn():
+            content = Application.get_textarea(self.result_textarea)
+            self.root.clipboard_clear()
+            self.root.clipboard_append(content)
+            self.root.update()
 
-            title = '<<PASTE - Clipboard>>'
-            self.set_textarea(self.textarea, data, title=title)
+        def callback_paste_text_btn():
+            try:
+                data = self.root.clipboard_get()
+                if not data:
+                    return
+
+                self.test_data = data
+
+                title = '<<PASTE - Clipboard>>'
+                self.set_textarea(self.textarea, data, title=title)
+            except Exception as ex:     # noqa
+                create_msgbox(
+                    title='Empty Clipboard',
+                    info='CAN NOT paste because there is no data in pasteboard.'
+                )
 
         def callback_snippet_btn():
-            create_msgbox(
-                title='TODO item',
-                info="TODO - Need to implement a function for snippet button"
-            )
+            user_data = Application.get_textarea(self.textarea)
+            if not user_data:
+                create_msgbox(
+                    title='Empty Data',
+                    error="Can NOT build Python test script without data."
+                )
+                return
+
+            try:
+                kwargs = self.get_pattern_args()
+                factory = RegexBuilder(
+                    user_data=user_data, test_data=self.test_data, **kwargs
+                )
+
+                kwargs = self.get_builder_args()
+                script = factory.generate_python_test(**kwargs)
+                self.set_textarea(self.result_textarea, script)
+                self.save_as_btn.config(state=tk.NORMAL)
+                self.copy_text_btn.config(state=tk.NORMAL)
+            except Exception as ex:
+                error = '{}: {}'.format(type(ex).__name__, ex)
+                create_msgbox(title='RegexBuilder Error', error=error)
 
         def callback_unittest_btn():
-            create_msgbox(
-                title='TODO item',
-                info="TODO - Need to implement a function for unittest button"
-            )
+            user_data = Application.get_textarea(self.textarea)
+            if not user_data:
+                create_msgbox(
+                    title='Empty Data',
+                    error="Can NOT build Python Unittest script without data."
+                )
+                return
+
+            try:
+                kwargs = self.get_pattern_args()
+                factory = RegexBuilder(
+                    user_data=user_data, test_data=self.test_data, **kwargs
+                )
+
+                kwargs = self.get_builder_args()
+                script = factory.generate_unittest(**kwargs)
+                self.set_textarea(self.result_textarea, script)
+                self.save_as_btn.config(state=tk.NORMAL)
+                self.copy_text_btn.config(state=tk.NORMAL)
+            except Exception as ex:
+                error = '{}: {}'.format(type(ex).__name__, ex)
+                create_msgbox(title='RegexBuilder Error', error=error)
 
         def callback_pytest_btn():
-            create_msgbox(
-                title='TODO item',
-                info="TODO - Need to implement a function for pytest button"
-            )
+            user_data = Application.get_textarea(self.textarea)
+            if not user_data:
+                create_msgbox(
+                    title='Empty Data',
+                    error="Can NOT build Python Pytest script without data."
+                )
+                return
 
-        def callback_rf_btn():
-            create_msgbox(
-                title='TODO item',
-                info="TODO - Need to implement a function for Robotframework button"
-            )
+            try:
+                kwargs = self.get_pattern_args()
+                factory = RegexBuilder(
+                    user_data=user_data, test_data=self.test_data, **kwargs
+                )
+
+                kwargs = self.get_builder_args()
+                script = factory.generate_pytest(**kwargs)
+                self.set_textarea(self.result_textarea, script)
+                self.save_as_btn.config(state=tk.NORMAL)
+                self.copy_text_btn.config(state=tk.NORMAL)
+            except Exception as ex:
+                error = '{}: {}'.format(type(ex).__name__, ex)
+                create_msgbox(title='RegexBuilder Error', error=error)
+
+        # def callback_rf_btn():
+        #     create_msgbox(
+        #         title='Robotframework feature',
+        #         info="Robotframework button is available in Pro or Enterprise Edition."
+        #     )
 
         # radio buttons
         self.line_radio_btn = ttk.Radiobutton(
-            self.entry_frame, text='line', variable=self.radio_btn_var,
+            self.entry_frame, text='line',
+            variable=self.radio_line_or_block_btn_var,
             value='line'
         )
         self.line_radio_btn.place(x=10, y=10)
 
         self.block_radio_btn = ttk.Radiobutton(
-            self.entry_frame, text='block', variable=self.radio_btn_var,
+            self.entry_frame, text='block',
+            variable=self.radio_line_or_block_btn_var,
             value='block'
         )
         self.block_radio_btn.place(x=55, y=10)
-        self.block_radio_btn.invoke()
 
         # open button
         open_file_btn = ttk.Button(self.entry_frame, text='Open',
-                                   command=self.callback_file_open, width=8)
+                                   command=self.callback_file_open, width=6)
         open_file_btn.place(x=110, y=10)
 
+        # Save As button
+        self.save_as_btn = ttk.Button(self.entry_frame, text='Save As',
+                                      command=callback_save_as_btn, width=7)
+        self.save_as_btn.place(x=155, y=10)
+        self.save_as_btn.config(state=tk.DISABLED)
+
+        # copy button
+        self.copy_text_btn = ttk.Button(self.entry_frame, text='Copy',
+                                        command=callback_copy_text_btn, width=6)
+        self.copy_text_btn.place(x=206, y=10)
+        self.copy_text_btn.config(state=tk.DISABLED)
+
         # paste button
         paste_text_btn = ttk.Button(self.entry_frame, text='Paste',
-                                    command=callback_paste_text_btn, width=8)
-        paste_text_btn.place(x=170, y=10)
+                                    command=callback_paste_text_btn, width=6)
+        paste_text_btn.place(x=251, y=10)
 
         # clear button
         clear_text_btn = ttk.Button(self.entry_frame, text='Clear',
-                                    command=callback_clear_text_btn, width=8)
-        clear_text_btn.place(x=230, y=10)
+                                    command=callback_clear_text_btn, width=6)
+        clear_text_btn.place(x=296, y=10)
 
-        # run button
-        run_btn = ttk.Button(self.entry_frame, text='Run',
-                             command=callback_run_btn, width=8)
-        run_btn.place(x=290, y=10)
+        # build button
+        build_btn = ttk.Button(self.entry_frame, text='Build',
+                               command=callback_build_btn, width=6)
+        build_btn.place(x=341, y=10)
 
         # snippet button
         snippet_btn = ttk.Button(self.entry_frame, text='Snippet',
-                                 command=callback_snippet_btn, width=8)
-        snippet_btn.place(x=350, y=10)
+                                 command=callback_snippet_btn, width=7)
+        snippet_btn.place(x=386, y=10)
 
         # unittest button
         unittest_btn = ttk.Button(self.entry_frame, text='Unittest',
-                                  command=callback_unittest_btn, width=8)
-        unittest_btn.place(x=410, y=10)
+                                  command=callback_unittest_btn, width=7)
+        unittest_btn.place(x=437, y=10)
 
         # pytest button
         pytest_btn = ttk.Button(self.entry_frame, text='Pytest',
-                                command=callback_pytest_btn, width=8)
-        pytest_btn.place(x=470, y=10)
+                                command=callback_pytest_btn, width=6)
+        pytest_btn.place(x=488, y=10)
 
         # Robotframework button
-        rf_btn = ttk.Button(self.entry_frame, text='Robotframework',
-                            command=callback_rf_btn, width=16)
-        rf_btn.place(x=530, y=10)
+        # rf_btn = ttk.Button(self.entry_frame, text='RF',
+        #                     command=callback_rf_btn, width=4)
+        # rf_btn.place(x=533, y=10)
 
     def build_result(self):
         """Build result text"""
