@@ -3,12 +3,14 @@
 import re
 import yaml
 import string
+from textwrap import dedent
 from pathlib import Path, PurePath
 from regexapp.exceptions import EscapePatternError
 from regexapp.exceptions import PatternReferenceError
 from regexapp.exceptions import TextPatternError
 from regexapp.exceptions import ElementPatternError
 from regexapp.exceptions import LinePatternError
+from regexapp.exceptions import BlockPatternError
 from regexapp.exceptions import PatternBuilderError
 
 import logging
@@ -104,17 +106,21 @@ class VarCls:
 
 
 class PatternReference(dict):
-    """Use to load regular expression pattern from system_settings.yaml
-    or/and user_settings.yaml
+    """Use to load regular expression pattern from system_references.yaml
+    or/and user_references.yaml
 
     Attribute
     ---------
-    sys_ref_loc (str): a system settings file name.
-    user_ref_loc (str): a user settings file name.
+    sys_ref_loc (str): a system references file name.
+    user_ref_loc (str): a user references file name.
 
     Methods
     -------
     load_reference(filename) -> None
+    PatternReference.get_pattern_layout(name) -> str
+    is_valid_format(name, value) -> bool
+    is_violated(dict_obj) -> bool
+    test(self, content) -> bool
 
     Raises
     ------
@@ -122,17 +128,19 @@ class PatternReference(dict):
             an invalid format
     """
 
-    # regexp pattern - from system settings
-    sys_ref_loc = str(PurePath(Path(__file__).parent, 'system_settings.yaml'))
-    # regex patterns - from user settings
-    user_ref_loc = str(PurePath(Path.home(), '.regexapp', 'user_settings.yaml'))
+    # regexp pattern - from system references
+    sys_ref_loc = str(PurePath(Path(__file__).parent, 'system_references.yaml'))
+    # regex patterns - from user references
+    user_ref_loc = str(PurePath(Path.home(), '.regexapp', 'user_references.yaml'))
 
     def __init__(self):
         self.load_reference(self.sys_ref_loc)
         self.load_reference(self.user_ref_loc)
+        self.test_result = ''
+        self.violated_format = ''
 
     def load_reference(self, filename):
-        """Load reference from YAML settings file.
+        """Load reference from YAML references file.
         Parameters
         ----------
         filename (str): a file name.
@@ -181,6 +189,162 @@ class PatternReference(dict):
             msg = '{} - {}'.format(type(ex).__name__, ex)
             raise PatternReferenceError(msg)
 
+    @classmethod
+    def get_pattern_layout(cls, name):
+        layout1 = """
+            name_placeholder:
+              ##################################################################
+              # double back flash must be used in a value of format if needed
+              # positive test and/or negative test is/are optional
+              ##################################################################
+              group: "replace_me"
+              description: "replace_me"
+              pattern: "replace_me"
+              # positive test:
+              #   change this positive test name: "replace_me -> (string or list of string)"
+              # negative test:
+              #   change this negative test name: "replace_me -> (string or list of string)"
+        """
+        layout2 = """
+            name_placeholder:
+              ##################################################################
+              # double back flash must be used in a value of format if needed
+              # positive test and/or negative test is/are optional
+              ##################################################################
+              group: "replace_me"
+              description: "replace_me"
+              format: "replace_me"
+              # format1: "replace_me"
+              # format2: "replace_me"
+              # formatn: "replace_me"
+              # positive test:
+              #   change this positive test name: "replace_me -> (string or list of string)"
+              # negative test:
+              #   change this negative test name: "replace_me -> (string or list of string)"
+        """
+        layout1, layout2 = dedent(layout1).strip(), dedent(layout2).strip()
+        return layout2 if 'datetime' in name else layout1
+
+    def is_valid_format(self, name, value):
+        """Check if name and value are valid
+
+        Parameters
+        ----------
+        name (str): a name of reference
+        value (dict): a value of reference
+
+        Returns
+        -------
+        bool: True if a reference is valid.
+        """
+        fmt1 = 'value of "{}" MUST be a dictionary'
+        fmt2 = 'value of "{}" MUST have "group" key'
+        fmt3 = 'value of "{}" MUST have "description" key'
+        fmt4 = 'value of "positive test" of "{}" MUST be a dictionary'
+        fmt5 = 'value of "negative test" of "{}" MUST be a dictionary'
+        fmt6 = ('value of "{}" (i.e datetime reference) '
+                'MUST have "format" or "format#" key')
+        fmt7 = 'value of "{}" MUST have "pattern" key'
+
+        if not isinstance(value, dict):
+            self.violated_format = fmt1.format(name)
+            return False
+
+        if 'group' not in value:
+            self.violated_format = fmt2.format(name)
+            return False
+
+        if 'description' not in value:
+            self.violated_format = fmt3.format(name)
+            return False
+
+        if 'positive test' in value:
+            pos_test_value = value.get('positive test')
+            if not isinstance(pos_test_value, dict):
+                self.violated_format = fmt4.format(name)
+                return False
+
+        if 'negative test' in value:
+            neg_test_value = value.get('positive test')
+            if not isinstance(neg_test_value, dict):
+                self.violated_format = fmt5.format(name)
+                return False
+
+        if 'datetime' in name:
+            for key in value:
+                if 'format' in key:
+                    return True
+            self.violated_format = fmt6.format(name)
+            return False
+        else:
+            if 'pattern' in value:
+                return True
+            else:
+                self.violated_format = fmt7.format(name)
+                return False
+
+    def is_violated(self, dict_obj):
+        """Check if new pattern reference doesnt violate with system reference
+
+        Parameters
+        ----------
+        dict_obj (dict): a dict object.
+
+        Returns
+        -------
+        bool: True there is a violation.
+        """
+        sys_ref = yaml.load(open(self.sys_ref_loc), Loader=yaml.SafeLoader)
+        fmt = '{} is ALREADY existed in system_references.yaml'
+        for name in dict_obj:
+            if 'datetime' not in name:
+                if name in sys_ref:
+                    self.violated_format = fmt.format(name)
+                    return True
+        return False
+
+    def test(self, content):
+        """test pattern reference.
+        Parameters
+        ----------
+        content (str): a content of YAML format.
+
+        Returns
+        -------
+        bool: True if content is a valid format
+
+        Raises
+        ------
+        PatternReferenceError: raise exception if a content is
+                an invalid format or violate system_references.yaml
+        """
+
+        try:
+            yaml_obj = yaml.load(content, Loader=yaml.SafeLoader)
+        except Exception as ex:
+            msg = '{} - {}'.format(type(ex).__name__, ex)
+            raise PatternReferenceError(msg)
+
+        if not yaml_obj:
+            logger.warning('CANT test an empty content')
+            self.test_result = 'not_tested'
+            return True
+
+        if not isinstance(yaml_obj, dict):
+            msg = 'content must be structure of dictionary.'
+            raise PatternReferenceError(msg)
+
+        for name, value in yaml_obj.items():
+            if not self.is_valid_format(name, value):
+                raise PatternReferenceError(self.violated_format)
+
+        if self.is_violated(yaml_obj):
+            raise PatternReferenceError(self.violated_format)
+
+        self.test_result = 'tested'
+        return True
+
+
 REF = PatternReference()
 
 
@@ -743,7 +907,7 @@ class ElementPattern(str):
 
         table = dict(space=r'^ *', space_plus=r'^ +',
                      ws=r'^\s*', ws_plus=r'^\s+')
-        pat = table.get(params, r'^\s*')
+        pat = table.get(params, r'^ *')
         return True, pat
 
     @classmethod
@@ -764,7 +928,7 @@ class ElementPattern(str):
 
         table = dict(space=r' *$', space_plus=r' +$',
                      ws=r'\s*$', ws_plus=r'\s+$')
-        pat = table.get(params, r'\s*$')
+        pat = table.get(params, r' *$')
         return True, pat
 
     @classmethod
@@ -1034,12 +1198,12 @@ class LinePattern(str):
                 appended_ws=appended_ws, ignore_case=ignore_case
             )
         else:
-            pattern = r'^\s*$'
+            pattern = r'^[ \t\v]*$'
         return str.__new__(cls, pattern)
 
     def __init__(self, text, used_space=True,
-                prepended_ws=False, appended_ws=False,
-                ignore_case=False):
+                 prepended_ws=False, appended_ws=False,
+                 ignore_case=False):
         self.variables = self._variables
         self.items = self._items
 
@@ -1106,10 +1270,10 @@ class LinePattern(str):
                     lst.append(TextPattern(after_match, used_space=used_space))
 
         if len(lst) == 1 and lst[0].strip() == '':
-            return r'^\s*$'
+            return r'^[ \t\v]*$'
         elif not lst:
             if line.strip() == '':
-                return r'^\s*$'
+                return r'^[ \t\v]*$'
             lst.append(TextPattern(line, used_space=used_space))
 
         cls.readjust_if_or_empty(lst, used_space=used_space)
@@ -1240,6 +1404,111 @@ class LinePattern(str):
             lst.append('{}$'.format(ws_pat))
 
 
+class BlockPattern(str):
+    """Use to convert multiple lines to regex pattern
+
+    Parameters
+    ----------
+    text (str): a text.
+    ignore_case (bool): prepend (?i) at the beginning of a pattern.
+            Default is False.
+
+    """
+    def __new__(cls, text, ignore_case=False):
+
+        lines = []
+        if isinstance(text, (list, tuple)):
+            for line in text:
+                lines.extend(str(line).splitlines())
+        elif isinstance(text, str):
+            lines = text.splitlines()
+        else:
+            'text argument must be string or list of string'
+            raise BlockPatternError(text)
+
+        if lines:
+            pattern = cls.get_pattern(lines, ignore_case=ignore_case)
+        else:
+            pattern = r'^\s*$'
+        return str.__new__(cls, pattern)
+
+    @classmethod
+    def get_pattern(cls, lines, ignore_case=False):
+        """convert text to regex pattern
+
+        Parameters
+        ----------
+        lines (lines): list of string
+        ignore_case (bool): prepend (?i) at the beginning of a pattern.
+                Default is False.
+
+        Returns
+        -------
+        str: a regex pattern.
+
+        """
+
+        if not lines:
+            return r'^\s*$'
+
+        line_patterns = []
+        for line in lines:
+            line_pat = LinePattern(line, ignore_case=ignore_case)
+            line_patterns.append(line_pat)
+
+        first, last = line_patterns[0], line_patterns[-1]
+        last = line_patterns[-1]
+
+        if len(line_patterns) == 1:
+            return first
+
+        new_line_patterns = [cls.reformat(first, is_first=True)]
+        other_line_pat = r'([^\r\n]*[\r\n]+)*'
+        for line_pat in line_patterns[1:-1]:
+            new_line_patterns.append(other_line_pat)
+            new_line_patterns.append(cls.reformat(line_pat))
+
+        new_line_patterns.append(other_line_pat)
+        new_line_patterns.append(cls.reformat(last, is_last=True))
+
+        new_pattern = ''.join(new_line_patterns)
+        return new_pattern
+
+    @classmethod
+    def reformat(cls, pattern, is_first=False, is_last=False):
+        """reformat pattern to work with re.DOTALL matching
+
+        Parameters
+        ----------
+        pattern (LinePattern, str): a line pattern.
+        is_first (bool): indicator to tell that is a first line.  Default is False.
+        is_last (bool): indicator to tell that is a last line.  Default is False.
+
+        Returns
+        -------
+        str: a new pattern after reformat.
+        """
+        if is_first:
+            if pattern.startswith('(?i)'):
+                new_pattern = pattern.replace('(?i)', '(?is)')
+            else:
+                new_pattern = '(?s){}'.format(pattern)
+        else:
+            new_pattern = pattern.replace('(?i)', '')
+            if new_pattern.startswith('^'):
+                new_pattern = new_pattern[1:]
+            else:
+                new_pattern = r'[^\r\n]*{}'.format(new_pattern)
+
+        if not is_last:
+            if new_pattern.endswith('$'):
+                new_pattern = r'{}[\r\n]+'.format(new_pattern[:-1])
+            else:
+                new_pattern = r'{}[^\r\n]*[\r\n]+'.format(new_pattern)
+
+        return new_pattern
+
+
 class PatternBuilder(str):
     """Use to convert a list of text to regex pattern
 
